@@ -3,7 +3,7 @@ package Alien::Base::ModuleBuild;
 use strict;
 use warnings;
 
-our $VERSION = '0.004';
+our $VERSION = '0.005';
 $VERSION = eval $VERSION;
 
 use parent 'Module::Build';
@@ -17,6 +17,7 @@ use Archive::Extract;
 use Sort::Versions;
 use List::MoreUtils qw/uniq first_index/;
 use ExtUtils::Installed;
+use File::Copy qw/move/;
 
 use Alien::Base::PkgConfig;
 use Alien::Base::ModuleBuild::Cabinet;
@@ -65,7 +66,7 @@ __PACKAGE__->add_property( alien_selection_method => 'newest' );
 # alien_build_commands: arrayref of commands for building
 __PACKAGE__->add_property( 
   alien_build_commands => 
-  default => [ '%pconfigure --prefix=%s', 'make' ],
+  default => [ '%c --prefix=%s', 'make' ],
 );
 
 # alien_test_commands: arrayref of commands for testing the library
@@ -82,7 +83,7 @@ __PACKAGE__->add_property(
 );
 
 # alien_version_check: command to execute to check if install/version
-__PACKAGE__->add_property( alien_version_check => 'pkg-config --modversion %n' );
+__PACKAGE__->add_property( alien_version_check => Alien::Base::PkgConfig->pkg_config_command . ' --modversion %n' );
 
 # pkgconfig-esque info, author provides these by hand for now, will parse .pc file eventually
 __PACKAGE__->add_property( 'alien_provides_cflags' );
@@ -101,6 +102,9 @@ __PACKAGE__->add_property( 'alien_repository'         => {} );
 __PACKAGE__->add_property( 'alien_repository_default' => {} );
 __PACKAGE__->add_property( 'alien_repository_class'   => {} );
 
+# alien_isolate_dynamic
+__PACKAGE__->add_property( 'alien_isolate_dynamic' => 0 );
+__PACKAGE__->add_property( 'alien_autoconf_with_pic' => 1 );
 
 ################
 #  ConfigData  #
@@ -132,6 +136,17 @@ sub new {
   # setup additional temporary directories, and yes we have to add File::ShareDir manually
   $self->_add_prereq( 'requires', 'File::ShareDir', '1.00' );
 
+  if (grep /(?<!\%)\%c/, @{ $self->alien_build_commands }) {
+    $self->config_data( 'autoconf' => 1 );
+  }
+
+  if ($^O eq 'MSWin32' && $self->config_data( 'autoconf')) {
+    $self->_add_prereq( 'build_requires', 'Alien::MSYS', '0' );
+    $self->config_data( 'msys' => 1 );
+  } else {
+    $self->config_data( 'msys' => 0 );
+  }
+
   # force newest for all automated testing 
   #TODO (this probably should be checked for "input needed" rather than blindly assigned)
   if ($ENV{AUTOMATED_TESTING}) {
@@ -232,10 +247,11 @@ sub ACTION_alien_code {
 
     my $file = $cabinet->files->[0];
     $version = $file->version;
-    $self->config_data( version => $version ); # Temporary setting, may be overridden later
+    $self->config_data( alien_version => $version ); # Temporary setting, may be overridden later
 
     print "Downloading File: " . $file->filename . " ... ";
     my $filename = $file->get;
+    croak "Error downloading file" unless $filename;
     print "Done\n";
 
     print "Extracting Archive ... ";
@@ -243,7 +259,7 @@ sub ACTION_alien_code {
     $ae->extract;
     print "Done\n";
 
-    my $extract_path = $ae->extract_path;
+    my $extract_path = _catdir($ae->extract_path);
     $self->config_data( working_directory => $extract_path );
     $CWD = $extract_path;
 
@@ -307,8 +323,13 @@ sub ACTION_alien_install {
 
   return if $self->config_data( 'install_type' ) eq 'system';
 
+  my $destdir = $self->destdir;
+
   {
     my $target = $self->alien_library_destination;
+    # prefix the target directory with $destdir so that package builds
+    # can install into a fake root
+    $target = File::Spec->catdir($destdir, $target) if defined $destdir;
     local $CWD = $target;
 
     # The only form of introspection that exists is to see that the README file
@@ -321,10 +342,35 @@ sub ACTION_alien_install {
 
   {
     local $CWD = $self->config_data( 'working_directory' );
+    local $ENV{DESTDIR} = $ENV{DESTDIR};
+    $ENV{DESTDIR} = $destdir if defined $destdir;
     print "Installing library to $CWD ... ";
     $self->alien_do_commands('install') or die "Failed\n";
     print "Done\n";
   }
+  
+  if ( $self->alien_isolate_dynamic ) {
+    my $target = $self->alien_library_destination;
+    # prefix the target directory with $destdir so that package builds
+    # can install into a fake root
+    $target = File::Spec->catdir($destdir, $target) if defined $destdir;
+    local $CWD = $target;
+    print "Isolating dynamic libraries ... ";
+    mkdir 'dynamic' unless -d 'dynamic';
+    foreach my $dir (qw( bin lib )) {
+      next unless -d $dir;
+      opendir(my $dh, $dir);
+      my @dlls = grep { /\.so/ || /\.(dylib|la|dll|dll\.a)$/ } grep !/^\./, readdir $dh;
+      closedir $dh;
+      foreach my $dll (@dlls) {
+        my $from = File::Spec->catfile($dir, $dll);
+        my $to   = File::Spec->catfile('dynamic', $dll);
+        unlink $to if -e $to;
+        move($from, $to);
+      }
+    }
+    print "Done\n";
+  }
 
   # refresh metadata after library installation
   $self->alien_refresh_manual_pkgconfig( $self->alien_library_destination );
@@ -426,7 +472,7 @@ sub alien_library_destination {
     : $self->install_destination('lib');
 
   my $dist_name = $self->dist_name;
-  my $dest = File::Spec->catdir( $lib_dir, qw/auto share dist/, $dist_name );
+  my $dest = _catdir( $lib_dir, qw/auto share dist/, $dist_name );
   return $dest;
 }
 
@@ -459,6 +505,18 @@ sub alien_detect_blib_scheme {
 #  Build Methods  #
 ###################
 
+sub _msys_do_system {
+  my $self = shift;
+  my $command = shift;
+  
+  if ($self->config_data( 'msys' )) {
+    require Alien::MSYS;
+    return Alien::MSYS::msys(sub { $self->do_system( $command ) });
+  }
+  
+  $self->do_system( $command );
+}
+
 sub alien_do_commands {
   my $self = shift;
   my $phase = shift;
@@ -468,7 +526,7 @@ sub alien_do_commands {
 
   foreach my $command (@$commands) {
 
-    my %result = $self->do_system( $command );
+    my %result = $self->_msys_do_system( $command );
     unless ($result{success}) {
       carp "External command ($result{command}) failed! Error: $?\n";
       return 0;
@@ -516,6 +574,7 @@ sub alien_interpolate {
   my ($string) = @_;
 
   my $prefix = $self->alien_exec_prefix;
+  my $configure = $self->alien_configure;
   my $share  = $self->alien_library_destination;
   my $name   = $self->alien_name || '';
 
@@ -524,6 +583,8 @@ sub alien_interpolate {
   $string =~ s/(?<!\%)\%s/$share/g;
   #   local exec prefix (ph: %p)
   $string =~ s/(?<!\%)\%p/$prefix/g;
+  #   correct incantation for configure on platform
+  $string =~ s/(?<!\%)\%c/$configure/g;
   #   library name (ph: %n)
   $string =~ s/(?<!\%)\%n/$name/g;
   #   current interpreter ($^X) (ph: %x)
@@ -533,7 +594,7 @@ sub alien_interpolate {
   # Version, but only if needed.  Complain if needed and not yet
   # stored.
   if ($string =~ /(?<!\%)\%v/) {
-    my $version = $self->config_data( 'version' );
+    my $version = $self->config_data( 'alien_version' );
     if ( ! defined( $version ) ) {
       carp "Version substution requested but unable to identify";
     } else {
@@ -556,6 +617,20 @@ sub alien_exec_prefix {
   }
 }
 
+sub alien_configure {
+  my $self = shift;
+  my $configure;
+  if ($self->config_data( 'msys' )) {
+    $configure = 'sh configure';
+  } else {
+    $configure = './configure';
+  }
+  if ($self->alien_autoconf_with_pic) {
+    $configure .= ' --with-pic';
+  }
+  $configure;
+}
+
 ########################
 #  Post-Build Methods  #
 ########################
@@ -563,7 +638,7 @@ sub alien_exec_prefix {
 sub alien_load_pkgconfig {
   my $self = shift;
 
-  my $dir = $self->config_data( 'working_directory' );
+  my $dir = _catdir($self->config_data( 'working_directory' ));
   my $pc_files = $self->rscan_dir( $dir, qr/\.pc$/ );
 
   my %pc_objects = map { 
@@ -594,12 +669,13 @@ sub alien_refresh_manual_pkgconfig {
 
 sub alien_generate_manual_pkgconfig {
   my $self = shift;
-  my ($dir) = @_;
+  my ($dir) = _catdir(shift);
 
   my $paths = $self->alien_find_lib_paths($dir);
 
   my @L = 
-    map { File::Spec->catdir( '-L${pcfiledir}', $_ ) }
+    map { "-L$_" }
+    map { _catdir( '${pcfiledir}', $_ ) }
     @{$paths->{lib}};
 
   my $provides_libs = $self->alien_provides_libs;
@@ -613,7 +689,8 @@ sub alien_generate_manual_pkgconfig {
   my $libs = join( ' ', @L, $provides_libs );
 
   my @I = 
-    map { File::Spec->catdir( '-I${pcfiledir}', $_ ) }
+    map { "-I$_" }
+    map { _catdir( '${pcfiledir}', $_ ) }
     @{$paths->{inc}};
 
   my $provides_cflags = $self->alien_provides_cflags;
@@ -654,7 +731,7 @@ sub alien_find_lib_paths {
   my @files =     
     map { File::Spec->abs2rel( $_, $dir ) }  # make relative to $dir
     grep { ! -d }
-    @{ $self->rscan_dir( $dir, $file_pattern ) };
+    @{ $self->_rscan_destdir( $dir, $file_pattern ) };
 
   my (@so_files, @lib_paths, @inc_paths);
   for (@files) {
@@ -692,12 +769,25 @@ sub alien_refresh_packlist {
   my $self = shift;
   my $dir = shift || croak "Must specify a directory to include in packlist";
 
-  my $inst = ExtUtils::Installed->new;
+  return unless $self->create_packlist;
+
+  my %installed_args;
+  $installed_args{extra_libs} = [map { File::Spec->catdir($self->destdir, $_) } @INC]
+    if defined $self->destdir;
+
+  my $inst = ExtUtils::Installed->new( %installed_args );
   my $packlist = $inst->packlist( $self->module_name );
   print "Using " .  $packlist->packlist_file . "\n";
 
   my $changed = 0;
-  my $files = $self->rscan_dir($dir);
+  my $files = $self->_rscan_destdir($dir);
+  # This is kind of strange, but MB puts the destdir paths in the
+  # packfile, when arguably it should not.  Usually you will want 
+  # to turn off packlists when you you are building an rpm anyway,
+  # but for the sake of maximum compat with MB we add the destdir
+  # back in after _rscan_destdir has stripped it out.
+  $files = [ map { File::Spec->catdir($self->destdir, $_) } @$files ]
+    if defined $self->destdir;
   for my $file (@$files) {
     next if $packlist->{$file};
     print "Adding $file to packlist\n"; 
@@ -708,6 +798,27 @@ sub alien_refresh_packlist {
   $packlist->write if $changed;
 }
 
+sub _rscan_destdir {
+  my($self, $dir, $pattern) = @_;
+  my $destdir = $self->destdir;
+  $dir = _catdir($destdir, $dir) if defined $destdir;
+  my $files = $self->rscan_dir($dir, $pattern);
+  $files = [ map { s/^$destdir//; $_ } @$files ] if defined $destdir;
+  $files;
+}
+
+# File::Spec uses \ as the file separator on MSWin32, which makes sense
+# since it is the default "native" file separator, but in practice / is
+# supported everywhere that matters and is significantly less problematic
+# in a number of common use cases (e.g. shell quoting).  This is a short
+# cut _catdir for this rather common pattern where you want catdir with
+# / as the file separator on Windows.
+sub _catdir {
+  my $dir = File::Spec->catdir(@_);
+  $dir =~ s{\\}{/}g if $^O eq 'MSWin32';
+  $dir;
+}
+
 1;
 
 __END__
@@ -719,8 +830,60 @@ Alien::Base::ModuleBuild - A Module::Build subclass for building Alien:: modules
 
 =head1 SYNOPSIS
 
+In your Build.PL:
+
+ use Alien::Base::ModuleBuild;
+ 
+ my $builder = Alien::Base::Module::Build->new(
+   module_name => 'Alien::MyLibrary',
+   
+   configure_requires => {
+     'Alien::Base' =>   '0.005',
+     'Module::Build' => '0.28'
+   },
+   requires => {
+     'Alien::Base' => '0.005',
+   },
+   
+   alien_name => 'mylibrary', # the pkg-config name if you want
+                              # to use pkg-config to discover
+                              # system version of the mylibrary
+   
+   alien_repository => {
+     protocol => 'http',
+     host     => 'myhost.org',
+     location => '/path/to/tarballs',
+     pattern  => qr{^mylibrary-([0-9\.]+)\.tar\.gz$},
+   },
+   
+   # this is the default:
+   alien_build_commands => [
+     "%c --prefix=%s", # %c is a platform independent version of ./configure
+     "make",
+   ],
+   
+   # this is the default for install:
+   alien_install_commands => [
+     "make install",
+   ],
+   
+   alien_isolate_dynamic => 1,
+ );
+
 =head1 DESCRIPTION
 
+This is a subclass of L<Module::Build>, that with L<Alien::Base> allows
+for easy creation of Alien distributions.  This module is used during the
+build step of your distribution.  When properly configured it will
+
+=over 4
+
+=item use pkg-config to find and use the system version of the library
+
+=item download, build and install the library if the system does not provide it
+
+=back
+
 =head1 GUIDE TO DOCUMENTATION
 
 The documentation for C<Module::Build> is broken up into sections:
@@ -784,7 +947,7 @@ L<Alien::Base>
 
 =head1 SOURCE REPOSITORY
 
-L<http://github.com/jberger/Alien-Base>
+L<http://github.com/Perl5-Alien/Alien-Base>
 
 =head1 AUTHOR
 
