@@ -3,12 +3,13 @@ package Alien::Base::PkgConfig;
 use strict;
 use warnings;
 
-our $VERSION = '0.004';
+our $VERSION = '0.005';
 $VERSION = eval $VERSION;
 
 use Carp;
 use File::Basename qw/fileparse/;
 use File::Spec;
+use Capture::Tiny qw( capture_stderr );
 
 sub new {
   my $class   = shift;
@@ -26,6 +27,7 @@ sub new {
   my ($name, $dir) = fileparse $path, '.pc';
 
   $dir = File::Spec->catdir( $dir );  # remove trailing slash
+  $dir =~ s{\\}{/}g;
 
   my $self = {
     package  => $name,
@@ -116,5 +118,22 @@ sub keyword {
   return $self->_interpolate_vars( $self->{keywords}{$keyword}, $override );
 }
 
+my $pkg_config_command;
+
+sub pkg_config_command {
+  unless (defined $pkg_config_command) {
+    capture_stderr {
+      if (`pkg-config --version` && $? == 0) {
+        $pkg_config_command = 'pkg-config';
+      } else {
+        require PkgConfig;
+        $pkg_config_command = "$^X $INC{'PkgConfig.pm'}";
+      }
+    }
+  }
+  
+  $pkg_config_command;
+}
+
 1;
 
